/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wFpeLSFunction.h"
using namespace tbox;
using namespace dart;

FpeLSFunction::FpeLSFunction(dart::Newton &_solver, std::vector<double> &_u,
                             std::vector<double> &_u0, Eigen::Map<Eigen::VectorXd> &_du,
                             Eigen::Map<Eigen::VectorXd> &_fRes) : LSFunction(), solver(_solver), u(_u), u0(_u0), du(_du), fRes(_fRes)
{
}

/**
 * @brief Update initial and change in solution 
 * @authors Adrien Crovato
 */
void FpeLSFunction::update(std::vector<double> &_u0, Eigen::Map<Eigen::VectorXd> &_du)
{
    u0 = _u0;
    du = _du;
}

/**
 * @brief Evaluate residual
 * @authors Adrien Crovato
 */
double FpeLSFunction::eval(double alpha)
{
    for (size_t i = 0; i < u.size(); ++i)
        u[i] = u0[i] + alpha * du(i);
    solver.buildRes(fRes);
    return fRes.norm();
}

void FpeLSFunction::write(std::ostream &out) const
{
    out << "dart::FpeLSFunction " << std::endl;
}
