/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WFREESTREAM_H
#define WFREESTREAM_H

#include "dart.h"
#include "wGroup.h"
#include "wF1El.h"

namespace dart
{

/**
 * @brief Manage freestream (Neumann) boundary condtion
 * @authors Adrien Crovato
 */
class DART_API Freestream : public tbox::Group
{
public:
#ifndef SWIG
    F1ElVi *f; ///< vector valued function to compute flux
#endif
    Freestream(std::shared_ptr<tbox::MshData> _msh, int no, int dim, double alpha, double beta = 0.0);
    Freestream(std::shared_ptr<tbox::MshData> _msh, std::string const &name, int dim, double alpha, double beta = 0.0);
    virtual ~Freestream();

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace dart

#endif //WFREESTREAM_H
